<?php
/*
Plugin Name: Download ePub
Plugin URI: http://www.karak.jp/
Description: エントリをePub形式でダウンロード
Version: 0.1
Author: Shinsuke Takeuchi
Author URI: http://www.karak.jp/
*/
/*****************************************/
/*
	*使い方*
	"Download ePub" というリンクを挿入したい場所に
	<?php if(isset($ePub)) $ePub->ePublisher(); ?>
	を挿入してください。
*/
require_once "File/Archive.php";
if(class_exists('WpDownloadePub')) {
	$ePub = & new WpDownloadePub();
}

class WpDownloadePub {
	var $ePub_dir;
	var $ePub_url;
	var $ePub_author;
	var $ePub_publisher;
	var $ePub_language;
	var $ePub_anchor_template;

	/* Constructor */
	function WpDownloadePub() {
		$this->ePub_dir = get_option('upload_path') . "/epub";
		$this->ePub_url = get_option('siteurl') . '/wp-content/uploads/epub/';
		$this->ePub_author ='author';
		$this->ePub_publisher = 'publisher';
		$this->ePub_language = 'ja';
		$this->ePub_anchor_template = '<div class="epub"><a href="%s" target="_blank">Download ePub</a></div>';
		if (!file_exists($this->ePub_dir)){
			return "ePubディレクトリが見つかりませんでした。";
		}
	}
	/* insert link */
	function ePublisher() {
		global $post;
		#ePubが存在しなければ作成
		if (!file_exists($epub_filename)){
			if(!$this->generate_ePub()){
				return;
			}
		}
		$timestamp = $this->get_timestamp($post->post_modified);
		$epub_filename = $this->ePub_url . $post->ID . $timestamp . '.epub';
		$link = sprintf($this->ePub_anchor_template, $epub_filename);
		echo $link;
	}
	/*
	generate_ePub
		レンタルサーバではsafe modeの制限が問題になるため、mkdirなどを使用せず、
		あらかじめ用意したスケルトンにデータを格納してそれをzip圧縮する
	*/
	function generate_ePub() {
		global $post;
		$skeleton_dir = dirname(__FILE__) . '/skeleton';
		#toc.ncx
		$value['uid'] = get_bloginfo('url') .'_'. date('YmdHis');
		$value['pid'] = $post->ID;
		$value['title'] = $post->post_title;
		$value['prefix'] = "epub" . $post->ID;
		$toc_ncx_content = $this->capture_template(dirname(__FILE__) . "/toc.ncx.tmpl", $value);
		#content.opf
		$value['author'] = $this->ePub_author;
		$value['publisher'] = $this->ePub_publisher;
		$value['language'] = $this->ePub_language;
		$content_opf_content = $this->capture_template(dirname(__FILE__) . "/content.opf.tmpl", $value);
		#xhtmlの生成
		$value['content'] = apply_filters('the_content', $post->post_content);
		$value['content'] = strip_tags($value['content'], '<body><head><html><title><abbr><acronym><address><blockquote><br><cite><code><dfn><div><em><h1><h2><h3><h4><h5><h6><kbd><p><pre><q><samp><span><strong><var><a><dl><dt><dd><ol><ul><li><object><param><b><big><hr><i><small><sub><sup><tt><del><ins><bdo><caption><col><colgroup><table><tbody><td><tfoot><th><thead><tr><img><area><map><meta><style><link><base>');
		$pattern = '/<img(?:.+?)src="([^\"]+)"([^>]*)>/u';
		$value['content'] = preg_replace_callback($pattern, array($this,"replace_img_paths"), $value['content']);
		$xhtml_content = $this->capture_template(dirname(__FILE__) . "/xhtml.tmpl", $value);
		#ファイル生成（mimetype、container.xmlは内容が固定されているので生成しない）
		if(!file_put_contents($skeleton_dir.'/OEBPS/toc.ncx', $toc_ncx_content)) {echo "toc.ncxの保存に失敗しました<br />";}
		if(!file_put_contents($skeleton_dir.'/OEBPS/content.opf', $content_opf_content)) {echo "content.opfの保存に失敗しました<br />";}
		$xhtml_filename = $skeleton_dir . '/OEBPS/' . $value['prefix'] . '.xhtml';
		if(!file_put_contents($xhtml_filename, $xhtml_content)) {echo "XHTMLの保存に失敗しました<br />";}
		#圧縮
		$timestamp = $this->get_timestamp($post->post_modified);
		$ePubFileName = $this->ePub_dir . "/" . $post->ID . $timestamp . ".epub";
		File_Archive::extract(File_Archive::read($skeleton_dir), File_Archive::toArchive($ePubFileName, File_Archive::toFiles(), 'zip'));
		#一時ファイルの削除
		unlink($skeleton_dir.'/OEBPS/toc.ncx');
		unlink($skeleton_dir.'/OEBPS/content.opf');
		unlink($xhtml_filename);
		$this->unlink_dir_contents($skeleton_dir.'/OEBPS/images');
		return 1;
	}
	#imgタグのsrcを置換
	function replace_img_paths($matches){
		$img_filename = strrchr($matches[1], '/');
		$new_path = dirname(__FILE__) . '/skeleton/OEBPS/images' . $img_filename;
		file_put_contents($new_path, file_get_contents($matches[1]));
		return '<img src="images'. $img_filename . '" />';
	}

	#テンプレート展開（$_filename＝テンプレートファイル名）
	function expand_template($_filename, $_data = array()) {
		extract($_data);
		include $_filename;
	}
	function capture_template($_filename, $_data = array()) {
		ob_start();
		$this->expand_template($_filename, $_data);
		$result = ob_get_contents();
		ob_end_clean();
		return $result;
	}
	#文字列（post_modified）からタイムスタンプを得る
	function get_timestamp($t) {
		list($y, $m, $d, $h, $i, $s) = sscanf($t, "%d-%d-%d %d:%d:%d");
		return mktime($h, $i, $s, $m, $d, $y);
	}
	#フォルダ内のファイルを削除
	function unlink_dir_contents($dir) {
		if (!$handle = @opendir($dir)) {return;}
		while (false !== ($target = readdir($handle))) {
			if($target == '.' || $target == '..') {continue;}
			@unlink($dir . '/' . $target);
		}
		closedir($handle);
	}
}

function h($str) {
	return htmlspecialchars($str);
}

?>
